// Copyright (c) 2016, root.ext@gmail.com. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import '../lib/dna.dart';
import 'dart:io' as io;
import 'dart:typed_data';

@Library('Kernel32')
class Kernel32 {
  dynamic lib = new DynamicLibrary<Kernel32>();

  int GetCurrentProcess()
    { return lib.GetCurrentProcess(); }

  int GetProcessId(@Param(C_INT) int process)
    { return lib.GetProcessId(process); }

  int GetModuleFileNameA(@Param(C_INT) int process, @Param(C_STRING) @Out() Ref<String> imageFileName, @Param(C_INT) int size)
    { return lib.GetModuleFileNameA(process, imageFileName, size); }

  bool GetProcessWorkingSetSize(@Param(C_INT) int process, @Param(TYPEDDATA) TypedData minimumWorkingSetSize, @Param(TYPEDDATA) TypedData maximumWorkingSetSize)
    { return lib.GetProcessWorkingSetSize(process, minimumWorkingSetSize, maximumWorkingSetSize); }

  int GetLastError() { return lib.GetLastError(); }
}

void main(){
  Kernel32 kernel32 = new Kernel32();
  var process = kernel32.GetCurrentProcess();
  var processId = kernel32.GetProcessId(process);
  print('pid $processId = ${io.pid}');

  var min = new Uint32List(1);
  var max = new Uint32List(1);
  var result = kernel32.GetProcessWorkingSetSize(process, min, max);
  print('working set min ${min} max ${max} result $result error ${kernel32.GetLastError()}');

  var name = new Ref(new String.fromCharCodes(new List.filled(64, 0)));
  result = kernel32.GetModuleFileNameA(0, name, name.value.length);
  print('module name  ${name.value} result $result error ${kernel32.GetLastError()}');

}
