// Copyright (c) 2016, root.ext@gmail.com. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import '../lib/dna.dart';
import 'dart:typed_data';

@Struct()
class Process {
  @Field(C_INT)
  int id;

  @Field(C_INT)
  int parentId;
}

@Library('libc.so.6')
class Libc {
  dynamic lib = new DynamicLibrary<Libc>();

  int getpid() { return lib.getpid(); }
  int getuid() { return lib.getuid(); }
  int getgid() { return lib.getgid(); }

  @Param(C_INT)
  int getgroups (int count, @Param(LISTINT) @Out() Ref<List<int>> groups) { return lib.getgroups(count, groups); }

  int rand() { return lib.rand(); }
  int rand_r(@Param(TYPEDDATA) TypedData seedp) { return lib.rand_r(seedp); }
  void srand(@Param(C_UINT) int seed) { return lib.srand(seed); }
}

void main(){
  Libc libc = new Libc();
  var processId = libc.getpid();
  print('pid $processId');

  var userId = libc.getuid();
  print('user id ${userId}');

  var groupId = libc.getgid();
  print('group id ${groupId}');

  var n = libc.getgroups(0, new Ref(new List()));
  var groups = new Ref<List<int>>(new List<int>(n));
  libc.getgroups(n, groups);
  print('groups ${groups.value}');

  libc.srand(0xDEADBEEF);
  var rand = libc.rand();
  print('random $rand');

  var seed = new Uint32List.fromList([0xDEADBEEF]);
  var srand = libc.rand_r(seed);
  print('random with seed $srand');

}
