// Copyright (c) 2016, root.ext@gmail.com. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:typed_data';
import 'package:test/test.dart';
import '../lib/dna.dart';

void main() {

  group('struct tests', () {

    setUp(() {

    });

    tearDown(() {

    });

    test('typeddata pointer', () {
      var data = new Uint8List(10);
      var pointer1 = getPointer(data);
      var pointer2 = getPointer(data);
      expect(pointer1, equals(pointer2));
    });

    test('sizes', () {
      var sizes = getSizes();
      expect(sizes, isNotEmpty);
    });

    test('struct encode/decode', () {
      var st = new SumSt(1,2,3);
      var data = encodeStruct(st);
      expect(data, isNotEmpty);

      var emptySt = new SumSt(0,0,0);
      decodeStruct(data, emptySt);
      expect(emptySt.a, equals(1));
      expect(emptySt.b, equals(2));
      //FIXME expect(emptySt.c, equals(3));
    });

    test('call with struct pointer', () {
      var sums = new Sums();
      var res =  sums.sumStPtr(new SumSt(1,2,3));
      expect(res, equals(6));
    });

  });
}

@Library('libdna.so', 'dna.dll')
class Sums {
  dynamic lib = new DynamicLibrary<Sums>();

  int sumStPtr(@Param(C_STRUCT) SumSt st) { return lib.sumStPtr(st); }
}

@Struct()
class SumSt {
  @Field(C_CHAR)
  int a;
  @Field(C_INT)
  int c;
  @Field(C_CHAR)
  int b;

  SumSt(this.a, this.b, this.c);
}
